-------------------------------------------------------------
--	Filename:  IN_TEST.VHD
-- Author: Adam Kwiatkowski / MSS
--	Version: 1
--	Date last modified: 1/26/12
-- Inheritance: 	none
--
-- Description: test NPINS pin connector input side
--
-- Rev 1 1/26/12 AZ
-- Added generic for parametric number of tested input pins
---------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity IN_TEST is 
Generic (

	NPINS: integer := 30
		-- number of pins being tested 
	);
port ( 
	--// Clock
	CLK : in std_logic;		-- Main internal processing clock

	--// User Interface
	DATA_IN: in std_logic_vector(NPINS downto 1);
	DIFFERENCE_OUT: out std_logic_vector(NPINS downto 1);
		-- displays difference between 1 clk delayed data
		-- when not the expected value of 1
	ERROR_CNT: out std_logic_vector(7 downto 0);
		-- counts errors, modulo 256
		-- stable value indicates functional connectors
	ERROR : out std_logic;

	TPS: out std_logic_vector(10 downto 1)		-- test points
);
end entity;

architecture behavioral of IN_TEST is
--------------------------------------------------------
--      COMPONENTS
--------------------------------------------------------

--------------------------------------------------------
--     SIGNALS
--------------------------------------------------------

signal DATA_IN_D: std_logic_vector(NPINS downto 1) := (others => '0');
signal DATA_IN_D2: std_logic_vector(NPINS downto 1) := (others => '0');

signal ERROR_CNT_LOCAL: std_logic_vector(7 downto 0) := (others => '0');
signal DIFFERENCE: std_logic_vector(NPINS downto 1) := (others => '0');
signal DIFFERENCE_D: std_logic_vector(NPINS downto 1) := (others => '0');

--------------------------------------------------------
--      IMPLEMENTATION
--------------------------------------------------------
begin

MAIN_001: process(CLK)
begin
	if rising_edge(CLK) then
		DATA_IN_D <= DATA_IN;
		DATA_IN_D2 <= DATA_IN_D;
		DIFFERENCE_D <= DIFFERENCE;
		if(DIFFERENCE_D = 1) then
			ERROR <= '0';
		else
			ERROR <= '1';
			ERROR_CNT_LOCAL <= ERROR_CNT_LOCAL + 1;
			DIFFERENCE_OUT <= DIFFERENCE_D;
		end if;						
	end if;						
end process;
ERROR_CNT <= ERROR_CNT_LOCAL;

DIFFERENCE <= (DATA_IN_D - DATA_IN_D2);


TPS(1) <= DATA_IN_D(3);
TPS(2) <= '1' when (DATA_IN_D = 0) else '0';
--TPS(2) <= DIFFERENCE_D(2);
--TPS(2) <= '0' when ((DATA_IN_D2 - DATA_IN_D) = 1) else '1';
--TPS(2) <= '1' when (ABS(DATA_IN_D - DATA_COUNTER) /= 1) else '0';
--TPS(2) <= DAC_READY;

end behavioral;

