-------------------------------------------------------------
--	Filename:  DNA_ID.VHD
-- Author: 
--		Alain Zarembowitch / MSS, 
--	Version: 0
--	Date last modified: 12-13-09
-- Inheritance: 	none
--
-- description:  Read the 57-bit unique ID
-- DESIGN IS NOT YET TESTED.. AWAITING PROTOTYPE BOARD
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
library UNISIM;
use UNISIM.VComponents.all;

entity DNA_ID is
    port ( 
			--// Clocks Resets
			CLK: in std_logic;
			ASYNC_RESET: in std_logic;

			--// Output
			ID_OUT: out std_logic_vector(56 downto 0);
			ID_VALID_OUT: out std_logic;
				-- '1' when the ID is valid
			ID_SAMPLE_CLK_OUT: out std_logic
				-- 1 CLK wide pulse when the ID is ready (one time after power up or reset)
			
			);
end entity;

architecture Behavioral of DNA_ID is
--------------------------------------------------------
--     SIGNALS
--------------------------------------------------------
-- Suffix _D indicates a one clock delay

signal STATE: integer range 0 to 60;
signal ID: std_logic_vector(56 downto 0);
signal DNA_READ: std_logic;
signal DNA_SHIFT: std_logic;
signal DNA_DOUT: std_logic;

--------------------------------------------------------
--      IMPLEMENTATION
--------------------------------------------------------
begin

-- state machine. counts from 0 to 60
STATE_001: process(ASYNC_RESET, CLK)
begin
	if (ASYNC_RESET = '1') then
		STATE <= 0;
	elsif rising_edge(CLK) then
		if(STATE < 60) then
			STATE <= STATE + 1;
		end if;
	end if;
end process;

-- read the DNA
DNA_READ <= '1' when (STATE = 1) else '0';

SHIFT_GEN: process(CLK)
begin
	if falling_edge(CLK) then
		-- note the falling edge, as per Xilinx recommendation
		-- shift 55 times
		if(STATE = 2) then
			DNA_SHIFT <= '1';
		elsif(STATE = 57) then
			DNA_SHIFT <= '0';
		end if;
	end if;
end process;

DNA_PORT_inst : DNA_PORT
generic map (
	SIM_DNA_VALUE => X"0000000000000000"  -- Specifies a DNA value for simulation purposes (the actual value
													  -- will be specific to the particular device used).
)
port map (
	DOUT => DNA_DOUT,   -- 1-bit Serial shifted output data
	CLK => CLK,     -- 1-bit Clock input
	DIN => '0',     -- 1-bit Data input to the shift register
	READ => DNA_READ,   -- 1-bit Synchronous load of the shift register with the Device DNA data
	SHIFT => DNA_SHIFT  -- 1-bit Active high shift enable input
);

ID_001: process(CLK)
begin
	if rising_edge(CLK) then
		if (DNA_SHIFT = '1') or (STATE = 57) then
			ID(0) <= DNA_DOUT;
			ID(56 downto 1) <= ID(55 downto 0);
		end if;
	end if;
end process;

ID_002: process(ASYNC_RESET, CLK)
begin
	if (ASYNC_RESET = '1') then
		ID_VALID_OUT <= '0';
		ID_SAMPLE_CLK_OUT <= '0';
	elsif rising_edge(CLK) then
		if (STATE = 57) then
			-- just read last bit.
			ID_VALID_OUT <= '1';
			ID_SAMPLE_CLK_OUT <= '1';
		else
			ID_SAMPLE_CLK_OUT <= '0';
		end if;
	end if;
end process;

ID_OUT <= ID;



end Behavioral;

